/**************************************************************************************************/


/*
 * File: polygon.h
 * Author: Roman Lysecky
 * Date Created: 22 September 2011
 *
 * Description: Declaration of a singly-linked list representing a polygon.
 *
 * Copyright: Roman Lysecky, Electrical and Computer Engineering, University of Arizona, 2011.
 *
 */

/**************************************************************************************************/


#include "global.h"
#include "linesegment.h"

/**************************************************************************************************/


#ifndef POLYGON_H
#define POLYGON_H

/**************************************************************************************************/

typedef struct Polygon_ {
    LineSegment *head;
    int size;
} Polygon;

/**************************************************************************************************/

/* initialize polygon to empty list */
void polygon_init(Polygon *polygon);

/* 
 * destroys the list by freeing all memory allocated for each line segment returning the polygon
 * to an empty list
 */
void polygon_destroy(Polygon *polygon);

/* 
 * inserts new line segment specified by (x1,y1), (x2, y2) into list after the given list element 
 * location. If the list element location is NULL, inserts line segment at head of list 
 */
bool polygon_insert_segment_next(Polygon *polygon, LineSegment *location, 
                                 float x1, float y1, float x2, float y2);

/* 
 * removes the line segment after the the given list element location. If the list element location
 * is NULL, removes the head of the list
 */
bool polygon_remove_segment_next(Polygon *polygon, LineSegment *location);

/* returns a pointer to the line segment at the head of the list */
LineSegment *polygon_head(Polygon *polygon);

/* returns the number of line segments within the polygon */
int polygon_num_segments(Polygon *polygon);

/* returns pointer to next line segment within the list */
LineSegment *polygon_next_segment(LineSegment *segment);

/* 
 * checks to determine if the specified polygon is a valid closed form polygon in which all segments 
 * are connected to another line segment
 */
bool is_valid_polygon(Polygon *polygon);

/* prints list of line segments within polygon */
void polygon_display(Polygon *polygon);

/**************************************************************************************************/


#endif

/**************************************************************************************************/

